﻿Imports System.Runtime.CompilerServices, System.ComponentModel

Public Module Converters

    ''' <summary> Convertiert ein beliebiges Objekt in einen Zieldatentyp</summary>
    Public Function ConvertItem(item As Object, srcType As Type, destType As Type) As Object
        Dim srcConv = TypeDescriptor.GetConverter(srcType)
        If srcConv.CanConvertTo(destType) Then Return srcConv.ConvertTo(item, destType)
        Dim destConv = TypeDescriptor.GetConverter(destType)
        If destConv.CanConvertFrom(srcType) Then Return destConv.ConvertFrom(item)
        Return Nothing
    End Function

    ''' <summary> Convertiert ein beliebiges Objekt in einen Zieldatentyp</summary>
    Public Function ConvertItem(item As Object, destType As Type) As Object
        Return ConvertItem(item, item.GetType, destType)
    End Function

    ''' <summary> wandelt Zeilenumbrüche in einem String in das HTML-Format mit "br" um</summary>
    <Extension>
    Public Function ConvertNewLineToHtmlBr(unformattedString As String) As String
        Return unformattedString.Replace(Microsoft.VisualBasic.vbCrLf, "<br>").Replace(Microsoft.VisualBasic.vbCr, "<br>").Replace(Microsoft.VisualBasic.vbLf, "<br>").Replace(Environment.NewLine, "<br>")
    End Function

    ''' <summary> entfernt unnötige Leerzeichen/Zeilenumbrüche in einem String</summary>
    <Extension>
    Public Function convertTrimString(unformattedString As String) As String
        Return unformattedString.Trim({" "c, Tab, Cr, Lf}).Replace(Environment.NewLine, " ")
    End Function

    ''' <summary>Versucht ein Objekt in Double zu Convertieren. Lässt sich das Objekt nicht als Double abwandeln kommt eine entsprechende
    ''' Fehlermeldung und es wird 0 zurückgegeben</summary>
    Public Function ConvertToDouble(dblDouble As Object, Optional WithHint As Boolean = True) As Double
        If dblDouble Is DBNull.Value OrElse dblDouble Is String.Empty Then
            Return 0#
        Else
            Try
                Return CDbl(dblDouble)
            Catch ex As Exception
                If WithHint Then msgError($"{dblDouble} ist keine gültige Zahl!")
                Return 0#
            End Try
        End If
    End Function

    ''' <summary>Versucht ein Objekt in Integer zu Convertieren. Lässt sich das Objekt nicht als Integer abwandeln kommt eine entsprechende
    ''' Fehlermeldung und es wird 0 zurückgegeben</summary>
    Public Function ConvertToInteger(intInteger As Object, Optional WithHint As Boolean = True) As Integer
        If intInteger Is DBNull.Value OrElse intInteger Is String.Empty Then
            Return 0
        Else
            Try
                Return CInt(intInteger)
            Catch ex As Exception
                If WithHint Then msgError($"{intInteger} ist keine gültige Zahl!")
                Return 0
            End Try
        End If
    End Function

    ''' <summary>Versucht ein Objekt in Decimal zu Convertieren. Lässt sich das Objekt nicht als Decimal abwandeln kommt eine entsprechende
    ''' Fehlermeldung und es wird 0 zurückgegeben</summary>
    Public Function ConvertToDecimal(decDecimal As Object, Optional WithHint As Boolean = True) As Decimal
        If decDecimal Is DBNull.Value OrElse decDecimal Is String.Empty Then
            Return 0D
        Else
            Try
                Return CDec(decDecimal)
            Catch ex As Exception
                If WithHint Then msgError($"{decDecimal} ist keine gültige Decimalzahl")
                Return 0D
            End Try
        End If
    End Function

    Public Function ConvertToDate(dtDate As Object, Optional WithHint As Boolean = True) As Date
        If dtDate Is DBNull.Value OrElse dtDate Is String.Empty Then
            Return #1/1/1900#
        Else
            Try
                Return CDate(dtDate)
            Catch ex As Exception
                If WithHint Then msgError($"{dtDate} ist keine gültige Datumsangabe")
                Return #1/1/1900#
            End Try
        End If
    End Function

    ''' <summary>Versucht, ein Objekt in Boolean zu Convertieren. Schlägt dies fehl, wird FALSE zurückgegeben</summary>
    Public Function ConvertToBoolean(blBool As Object) As Boolean
        If blBool Is Nothing Then Return False
        Select Case blBool.ToString.ToLower
            Case "true" : Return True
            Case "ja" : Return True
            Case "false" : Return False
            Case "nein" : Return False
            Case Else : Return False
        End Select
    End Function

End Module
