﻿Imports System.Collections, System.Runtime.CompilerServices, System.Reflection, System.Diagnostics, System.Drawing

Public Module DataGridViewHelper

#Region "Columns"

    ''' <summary> Passt einmalig alle Column-Breiten auf den Inhalt des kompletten DGV an</summary>
    <Extension>
    Public Sub SetDgvColumnWidthAutosizeOnce(dgv As DataGridView)
        WaitCursor()
        For Each col In dgv.ColumnsX
            col.AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells
            Dim i = col.Width
            col.AutoSizeMode = DataGridViewAutoSizeColumnMode.NotSet
            col.Width = i
        Next
        DefaultCursor()
    End Sub

    ''' <summary> Passt einmalig alle Column-Breiten auf den Inhalt des kompletten DGV an</summary>
    Public Sub SetDgvColumnWidthAutosizeOnce(ParamArray dgvs As DataGridView())
        WaitCursor()
        For Each dgv In dgvs
            For Each col In dgv.ColumnsX
                col.AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells
                Dim i = col.Width
                col.AutoSizeMode = DataGridViewAutoSizeColumnMode.NotSet
                col.Width = i
            Next
        Next
        DefaultCursor()
    End Sub

    ''' <summary> Return die Default-Admin Columns, nach Prüfung ob diese auch vorhanden sind </summary>
    <Extension>
    Public Function GetDefaultAdminCols(dgv As DataGridView) As List(Of String)
        Dim lstAdminCol As New List(Of String) From {"ID", "IDDataGridViewTextBoxColumn", "Bankverbindung"}
        Dim lstNew As New List(Of String)
        For Each colName In lstAdminCol
            If colExists(dgv, colName) Then lstNew.Add(colName)
        Next
        'urspr. Liste durchlaufen und String rausschmeißen ging nicht (mit for each, fehler in aufzählung, mit i wurde das übersprungen!)
        Return lstNew
    End Function

    ''' <summary> Überprüft, ob eine Column anhand ColumnName exisitert </summary>
    Private Function colExists(dgv As DataGridView, colName As String) As Boolean
        For i = 0 To dgv.Columns.Count - 1
            If dgv.Columns(i).Name.ToLower = colName.ToLower Then
                'Debug.WriteLine($"DGV:{dgv.Columns(i).Name.ToLower} | aus List: {colName.ToLower} | Return True")
                Return True
            End If
        Next
        Return False
    End Function

    ''' <summary> zeigt bestimmte Columns aus lstColNames an und blendet alle anderen aus, außerdem werden die Col-Widths wiederhergestellt </summary>
    <Extension>
    Public Sub ShowOnlySelectedCols(dgv As DataGridView, lstColNames As List(Of String))
        For Each col As DataGridViewColumn In dgv.Columns
            Dim selected = lstColNames.FirstOrDefault(Function(x) x.LeftCut(";"c) = col.Name)
            If selected IsNot Nothing Then
                col.Visible = CBool(selected.RightCut(";"c).RightCut(","c))
                col.Width = CInt(selected.RightCut(";").LeftCut(","))
            Else
                col.Visible = False
            End If
        Next
    End Sub

    ''' <summary> Friert DGV-Columns ein, sodass diese nicht mit gescrollt werden </summary>
    <Extension>
    Public Sub FreezeCol(dgv As DataGridView, colName As String)
        For Each col As DataGridViewColumn In dgv.Columns
            If col.Name = colName OrElse col.HeaderText = colName Then col.Frozen = True
        Next
    End Sub

    ''' <summary> Friert DGV-Columns ein, sodass diese nicht mit gescrollt werden </summary>
    <Extension>
    Public Sub FreezeCol(dgv As DataGridView, col As DataGridViewColumn)
        col.Frozen = True
    End Sub

#End Region 'Columns

#Region "gehe zu Datum"

    ''' <summary> zeigt einen Dialog zur Datumseingabe an. Ist DialogResult.OK wird im DGV nach dem Datum gesucht und die gefundene Row markiert </summary>
    <Extension>
    Public Sub GeheZuDatum(dgv As DataGridView)
        Using dlg As New dlgDateInput
            If dlg.ShowDialog() = DialogResult.OK Then dgv.GeheZuDatum(dlg.dtbDate.Text)
        End Using
    End Sub

    ''' <summary> sucht im DGV in der Spalte "Datum" nach dem Datum aus dlgDateInput. Wenn gefunden, wird die entsprechende Row markiert </summary>
    <Extension>
    Public Sub GeheZuDatum(dgv As DataGridView, Datum As String)
        Dim colDatum = 0
        For iCol = 0 To dgv.Columns.Count - 1
            If dgv.Columns(iCol).HeaderText = "Datum" Then
                colDatum = iCol
                Exit For
            End If
        Next
        For i = 0 To dgv.Rows.Count - 1
            If dgv.Rows(i).Cells(colDatum).FormattedValue.ToString = Datum Then
                With dgv
                    .FirstDisplayedScrollingRowIndex = If(i - 7 < 0, i, i - 7)
                    .Rows(i).Selected = True
                    For Each col In dgv.ColumnsX
                        If col.Visible Then
                            .CurrentCell = dgv.Rows(i).Cells(col.Index)
                            Exit For
                        End If
                    Next
                End With
                Exit For
            End If
        Next
    End Sub

#End Region 'gehe zu Datum

    ''' <summary> sucht im DGV nach dem angegebenen Datum und gibt den RowIndex zurück</summary>
    <Extension>
    Public Function HoleDatumRow(dgv As DataGridView, Datum As String) As Integer
        Dim colDatum = 0
        For iCol = 0 To dgv.Columns.Count - 1
            If dgv.Columns(iCol).HeaderText = "Datum" Then
                colDatum = iCol
                Exit For
            End If
        Next
        For i = 0 To dgv.Rows.Count - 1
            If dgv.Rows(i).Cells(colDatum).FormattedValue.ToString = Datum Then Return i
        Next
        Return 0
    End Function

#Region "DGV-Settings"

    Public Sub RegisterDgvSettings(ParamArray dgvs() As DataGridView)
        dgvs.ForEach(Sub(dgv) dgv.RegisterDgvSettings)
    End Sub

    ''' <summary>registriert alle relevanten Settings für das DGV, wenn DataBinding genutzt wird</summary>
    <Extension>
    Public Sub RegisterDgvSettings(dgv As DataGridView, Optional CustomCopyOrExport As Boolean = False)
        Dim frm = dgv.FindForm
        dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.None
        'zum Testen
        'dgv.DefaultCellStyle.WrapMode = DataGridViewTriState.True
        'dgv.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells
        '
        dgv.DGVDoubleBuffer
        RegisterDgvCellFormatting(dgv)
        registerClearSelection(frm)
        If CustomCopyOrExport Then
            AddHandler dgv.KeyDown, AddressOf DefaultKeyActionsWithoutCopyOrExport
        Else
            AddHandler dgv.KeyDown, AddressOf DefaultKeyActions
        End If
    End Sub

    ''' <summary>Allgemeiner DGV-KeyEventArgs-Handler für Standard-Aktionen</summary>
    Private Sub DefaultKeyActions(sender As Object, e As KeyEventArgs)
        Dim dgv = DirectCast(sender, DataGridView)
        If e.KeyCode = Keys.P AndAlso e.Modifiers = Keys.Control Then
            'Print
            e.SuppressKeyPress = True
        ElseIf e.KeyCode = Keys.C AndAlso e.Modifiers = Keys.Control Then
            dgv.CopyOrExportSelection(False, e)
            e.SuppressKeyPress = True
        ElseIf e.KeyCode = Keys.A AndAlso e.Modifiers = Keys.Control Then
            dgv.SelectAll()
            e.SuppressKeyPress = True
        ElseIf e.KeyCode = Keys.I AndAlso e.Modifiers = Keys.Control Then
            dgv.Info
            e.SuppressKeyPress = True
        End If
    End Sub

    Private Sub DefaultKeyActionsWithoutCopyOrExport(sender As Object, e As KeyEventArgs)
        Dim dgv = DirectCast(sender, DataGridView)
        If e.KeyCode = Keys.P AndAlso e.Modifiers = Keys.Control Then
            'Print
            e.SuppressKeyPress = True
        ElseIf e.KeyCode = Keys.A AndAlso e.Modifiers = Keys.Control Then
            dgv.SelectAll()
            e.SuppressKeyPress = True
        ElseIf e.KeyCode = Keys.I AndAlso e.Modifiers = Keys.Control Then
            dgv.Info
            e.SuppressKeyPress = True
        End If
    End Sub

#Region "CellFormatting"

    ''' <summary>Überschreibt die Ansicht mehrerer DGVs mit bestimmten Inhalten mit ""</summary>
    Public Sub RegisterDgvCellFormatting(ParamArray DGVs As DataGridView())
        For Each dgv In DGVs
            RemoveHandler dgv.CellFormatting, AddressOf DgvCellFormatting
            AddHandler dgv.CellFormatting, AddressOf DgvCellFormatting
        Next
    End Sub

    Private Sub DgvCellFormatting(sender As Object, e As DataGridViewCellFormattingEventArgs)
        Dim d As Date = Date.Parse("01.01.1900").Date
        If Object.Equals(e.Value, d) Then e.Value = "" : e.FormattingApplied = True
        If Object.Equals(e.Value, 0) Then e.Value = "" : e.FormattingApplied = True
        If Object.Equals(e.Value, 0D) Then e.Value = "" : e.FormattingApplied = True
        If e.Value?.GetType = GetType(Decimal) Then
            If CDec(e.Value) <> 0D Then e.Value = Math.Round(CDec(e.Value), 2).ToString : e.FormattingApplied = True
        End If
        If Object.Equals(e.Value, 0#) Then e.Value = "" : e.FormattingApplied = True
        If Object.Equals(e.Value, "<keine Angabe>") Then e.Value = "" : e.FormattingApplied = True
        If Object.Equals(e.Value, "<KEINE ANGABE>") Then e.Value = "" : e.FormattingApplied = True
        If Object.Equals(e.Value, "<keine Zuordnung>, <keine Zuordnung>") Then e.Value = "" : e.FormattingApplied = True
        If Object.Equals(e.Value, "<keine Angabe>, <keine Angabe>") Then e.Value = "" : e.FormattingApplied = True
    End Sub

#End Region 'CellFormatting

#Region "ClearSelection"

    ''' <summary>Löst ein "ClearSelection" bei Klick in's Leere aus</summary>
    Public Sub registerClearSelection(frm As Form)
        Dim getChilds As Func(Of Control, IEnumerable) = Function(ctl) ctl.Controls
        For Each ctl In getChilds.All(frm)
            Dim dgv = TryCast(ctl, DataGridView)
            If dgv.NotNull Then
                AddHandler dgv.MouseUp, AddressOf dgv_ClearSelection
            End If
        Next
    End Sub

    Private Sub dgv_ClearSelection(sender As Object, e As MouseEventArgs)
        Dim dgv = DirectCast(sender, DataGridView)
        If e.Button = MouseButtons.Left Then
            If dgv.HitTest(e.X, e.Y) Is DataGridView.HitTestInfo.Nowhere Then
                dgv.ClearSelection()
                dgv.CurrentCell = Nothing
            End If
        End If
    End Sub

#End Region 'ClearSelection

    ''' <summary>aktiviert das DoubleBuffering für ein DGV</summary>
    <Extension>
    Public Sub DGVDoubleBuffer(DGV As DataGridView)
        Dim dgvType As Type = DGV.GetType
        Dim pi As PropertyInfo = dgvType.GetProperty("DoubleBuffered", BindingFlags.Instance Or BindingFlags.NonPublic)
        pi.SetValue(DGV, True, Nothing)
    End Sub

#End Region 'DGV-Settings

#Region "Conditional Coloring"

    <Extension>
    Public Sub ConditionedDgvCellBackColor(dgv As DataGridView, RowIndex As Integer, condition As Boolean, col As DataGridViewColumn, Optional color As Color = Nothing, Optional OriginColor As Color = Nothing)
        If condition Then
            If color <> Nothing Then
                dgv.Rows(RowIndex).Cells(col.Index).Style.BackColor = color
            Else
                dgv.Rows(RowIndex).Cells(col.Index).Style.BackColor = Color.Red
            End If
        Else
            If OriginColor <> Nothing Then
                dgv.Rows(RowIndex).Cells(col.Index).Style.BackColor = OriginColor
            Else
                dgv.Rows(RowIndex).Cells(col.Index).Style.BackColor = SystemColors.Window
            End If
        End If
    End Sub

    <Extension>
    Public Sub ConditionedDgvCellBackColor(dgv As DataGridView, RowIndex As Integer, condition As Boolean, colName As String, Optional color As Color = Nothing, Optional OriginColor As Color = Nothing)
        If condition Then
            If color <> Nothing Then
                dgv.Rows(RowIndex).Cells(colName).Style.BackColor = color
            Else
                dgv.Rows(RowIndex).Cells(colName).Style.BackColor = Color.Red
            End If
        Else
            If OriginColor <> Nothing Then
                dgv.Rows(RowIndex).Cells(colName).Style.BackColor = OriginColor
            Else
                dgv.Rows(RowIndex).Cells(colName).Style.BackColor = SystemColors.Window
            End If
        End If
    End Sub

    <Extension>
    Public Sub ConditionedDgvCellBackColor(dgv As DataGridView, e As DataGridViewRowPrePaintEventArgs, condition As Boolean, colName As String, Optional color As Color = Nothing, Optional OriginColor As Color = Nothing)
        ConditionedDgvCellBackColor(dgv, e.RowIndex, condition, colName, color)
    End Sub

    <Extension>
    Public Sub ConditionedDgvRowBackColor(dgv As DataGridView, e As DataGridViewRowPrePaintEventArgs, condition As Boolean, Optional color As Color = Nothing)
        If condition Then
            If color <> Nothing Then
                dgv.Rows(e.RowIndex).DefaultCellStyle.BackColor = color
            Else
                dgv.Rows(e.RowIndex).DefaultCellStyle.BackColor = Color.Red
            End If
        Else
            dgv.Rows(e.RowIndex).DefaultCellStyle.BackColor = SystemColors.Window
        End If
    End Sub

    <Extension>
    Public Sub ConditionedDgvRowForeColor(dgv As DataGridView, RowIndex As Integer, condition As Boolean)
        If Not RowIndex < 0 AndAlso Not RowIndex > dgv.Rows.Count - 1 Then dgv.Rows(RowIndex).DefaultCellStyle.ForeColor = If(condition, Color.Red, SystemColors.ControlText)
    End Sub

    <Extension>
    Public Sub ConditionedDgvRowForeColor(dgv As DataGridView, e As DataGridViewRowPrePaintEventArgs, condition As Boolean)
        ConditionedDgvRowForeColor(dgv, e.RowIndex, condition)
    End Sub

#End Region 'Conditional Coloring

    <Extension>
    Public Function SelectedRowsX(dgv As DataGridView) As List(Of DataGridViewRow)
        Return dgv.SelectedRows.Cast(Of DataGridViewRow).ToList
    End Function

    <Extension>
    Public Function SelectedCellsX(dgv As DataGridView) As List(Of DataGridViewCell)
        Return dgv.SelectedCells.Cast(Of DataGridViewCell).ToList
    End Function

    <Extension()>
    Public Function CellsX(dgvR As DataGridViewRow) As List(Of DataGridViewCell)
        Return dgvR.Cells.Cast(Of DataGridViewCell).ToList
    End Function

    <Extension()>
    Public Function ColumnsX(dgv As DataGridView) As List(Of DataGridViewColumn)
        Return dgv.Columns.Cast(Of DataGridViewColumn).ToList
    End Function

    <Extension>
    Public Sub ShowSelectedRowsCounterInToolStripLabel(dgv As DataGridView, tssl As ToolStripLabel)
        tssl.Text = $"markierte Einträge: {dgv.SelectedRows.Count}"
    End Sub

    <Extension>
    Public Sub ShowTotalRowsCounterInToolStripLabel(dgv As DataGridView, tssl As ToolStripLabel)
        tssl.Text = $"Einträge: {dgv.Rows.Count}"
    End Sub

    ''' <summary> zeigt für das Debugging ein Dialog mit DGV mit den "Zwischendaten" an </summary>
    <Extension>
    Public Sub showDebugDGV(dt As DataTable, Optional Title As String = "")
        Using dlg As New dlgDebugDGV
            dlg.dgv.DataSource = dt
            dlg.dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.AllCells
            If Not Title = "" Then dlg.Text = Title
            dlg.ShowDialog()
        End Using
    End Sub

    ''' <summary> Speichert die Indizes der selektierten DGV-Cells in ein Dictionary(Of Integer, Integer) 
    ''' Left = RowIndex, Right = ColumnIndex</summary>
    <Extension>
    Public Function storeDgvSelectedCells(dgv As DataGridView) As List(Of String)
        Dim dicCells As New List(Of String)
        For Each selCell As DataGridViewCell In dgv.SelectedCells
            dicCells.Add($"{selCell.RowIndex}.{selCell.ColumnIndex}")
        Next
        Return dicCells
    End Function

    ''' <summary> Ruft die Indizes von vorher selektierten DGV-Cells aus einem Dictionary ab und stellt diese wieder her </summary>
    <Extension>
    Public Sub restoreDgvSelectedCells(dgv As DataGridView, dic As List(Of String))
        For Each itm In dic
            dgv.Item(ConvertToInteger(itm.RightCut("."c)), ConvertToInteger(itm.LeftCut("."c))).Selected = True
        Next
    End Sub

    ''' <summary> Speichert die Indizes der selektierten DGV-Rows in eine List(Of Integer) </summary>
    <Extension>
    Public Function storeDgvSelectedRows(dgv As DataGridView) As List(Of Integer)
        Dim lstRows As New List(Of Integer)
        For Each rw As DataGridViewRow In dgv.SelectedRows
            lstRows.Add(rw.Index)
        Next
        Return lstRows
    End Function

    ''' <summary> Ruft die Indizes von vorher selektierten DGV-Rows aus einer Liste ab und stellt diese wieder her </summary>
    <Extension>
    Public Sub restoreDgvSelectedRows(dgv As DataGridView, lst As List(Of Integer))
        For Each itm In lst
            If dgv.Rows.Count >= lst.Count Then dgv.Rows(itm).Selected = True
        Next
    End Sub

    ''' <summary> zeigt einen Info-Dialog mit Gesamt- und markierten Einträgen eines DataGridView </summary>
    <Extension>
    Public Sub Info(DGV As DataGridView)
        Using dlg As New dlgInfo
            Dim gesamt = DGV.Rows.Count
            Dim markiert = DGV.SelectedRows.Count
            dlg.lblGesamt.Text = $"Gesamt: {gesamt}"
            dlg.lblMarkiert.Text = $"markiert: {markiert}"
            dlg.ShowDialog()
        End Using
    End Sub

    ''' <summary>Ermöglicht das Umschalten des Edit-Modus eines DGV (einzelne Zellen oder ganze Rows) und setzt dabei CheckState(s) von ToolStripMenuItems</summary>
    <Extension>
    Public Sub FastEdit(DGV As DataGridView, ParamArray tsFastEdit As ToolStripMenuItem())
        If DGV.ReadOnly = True Then
            DGV.ReadOnly = False
            DGV.SelectionMode = DataGridViewSelectionMode.CellSelect
            For Each ts In tsFastEdit
                ts.Checked = True
            Next
        Else
            DGV.ReadOnly = True
            DGV.SelectionMode = DataGridViewSelectionMode.FullRowSelect
            For Each ts In tsFastEdit
                ts.Checked = False
            Next
        End If
    End Sub

    ''' <summary> Erweitertes Error-Handling für DataGridView </summary>
    Public Sub DgvErrorHandling(sender As Object, e As DataGridViewDataErrorEventArgs)
        Dim dgv = DirectCast(sender, DataGridView)
        If Not e.ColumnIndex = -1 Then
            Dim comboCol = TryCast(dgv.Columns(e.ColumnIndex), DataGridViewComboBoxColumn)
            If comboCol IsNot Nothing AndAlso e.Context = (DataGridViewDataErrorContexts.Formatting Or DataGridViewDataErrorContexts.Display) _
                AndAlso TypeOf e.Exception Is ArgumentException Then
                Dim bsPK = DirectCast(comboCol.DataSource, BindingSource)
                If bsPK Is Nothing Then
                    Throw New Exception($"{dgv.Name}: FEHLER! Sind alle Bindingsources auf Comboboxen gesetzt?")
                    Return
                End If
                Dim tbPK = bsPK.DataTable
                Dim defaultRow = tbPK.Rows.Cast(Of DataRow).FirstOrDefault(Function(rw) 0 = (rw.RowState And (DataRowState.Deleted Or DataRowState.Detached)))
                If defaultRow.NotNull Then
                    dgv(e.ColumnIndex, e.RowIndex).Value = defaultRow(comboCol.ValueMember) ' gültigen PK-Wert einsetzen
                    Return
                End If
            End If
        End If
        If TypeOf e.Exception Is InvalidConstraintException Then
            msgError($"Der Datensatz kann nicht gelöscht werden, da andere Datensätze mit diesem in Beziehung stehen.")
        Else
            Msg("ColumnIndex: ", e.ColumnIndex, ", Context: ", e.Context, Lf, e.Exception.Message)
        End If
    End Sub

    ''' <summary> Manuelles Row-Deleting-Handling eines DataGridViews </summary>
    <Extension>
    Public Sub rowDelete(dgv As DataGridView)
        If msgQuestionWarning("Die markierten Zeilen werden endgültig gelöscht, fortfahren?") = DialogResult.Yes Then
            Dim bs = DirectCast(dgv.DataSource, BindingSource)
            Dim selectedIndices = From rw In dgv.SelectedRows.Cast(Of DataGridViewRow)() Select rw.Index Order By Index Descending
            If Not selectedIndices.Any Then Return
            For Each indx In selectedIndices
                bs.RemoveAt(indx)
            Next
        End If
    End Sub

End Module


#Region "Highlighter"

'''<summary> Zum Highlighten von Treenodes, ListviewItems etc. </summary>
Public Class Highlighter
    ' Das Highlighten erfolgt durch Farb-Inversion des Target-Rectangles. Ausschalten
    ' des Highlights durch wiederholte Inversion desselben Rectangles. Fehl-Highlighting
    ' ergibt sich also, wenn zwischen Highlight() und Off() das Rectangle anderweitig
    ' übermalt wurde.  
    Protected Shared _HighlightedRect As Rectangle = Rectangle.Empty
    ' Farb-Inversion ist eine Art "Spiegelung" über eine Farb-Achse.
    ' Dieses komische Rot ergibt (bisher) immer deutliche "Spiegel-Farben" 
    Private Shared ReadOnly _AxisColor As Color = Color.FromArgb(255, 0, 127)
    Public Shared Sub Highlight(ByVal Target As Control, ByVal ItemRect As Rectangle)
        If ItemRect.IsEmpty Then
            Highlighter.Off()
        Else
            ItemRect.Intersect(Target.ClientRectangle)
            ' FillReversibleRectangle verwendet bildschirmbezogene Koordinaten, daher muß ItemRect
            ' um die Bildschirmposition des Controls verschoben werden
            ItemRect.Offset(Target.PointToScreen(Point.Empty))
            If ItemRect.IntersectsWith(_HighlightedRect) Then
                Return
            End If
            ' ist schon highlighted
            If Not _HighlightedRect.IsEmpty Then
                ' altes Highlight löschen
                ControlPaint.FillReversibleRectangle(_HighlightedRect, _AxisColor)
            End If
            _HighlightedRect = ItemRect
            ' neues Highlight setzen
            ControlPaint.FillReversibleRectangle(ItemRect, _AxisColor)
        End If
    End Sub
    Public Shared Sub Before(ByVal Target As Control, ByVal ItemRect As Rectangle)
        ItemRect.Offset(0, -ItemRect.Height \ 2)
        ItemRect.Inflate(0, -2)
        Highlight(Target, ItemRect)
    End Sub
    Public Shared Sub After(ByVal Target As Control, ByVal ItemRect As Rectangle)
        ItemRect.Offset(0, ItemRect.Height)
        Before(Target, ItemRect)
    End Sub
    Public Shared Sub FullWidth(ByVal Target As Control, ByVal Y As Integer, ByVal Height As Integer)
        'bei leeren ListenControls, oder bei Treeview.TopLevel als Ziel kann kein ItemRect angegeben werden
        Highlighter.Highlight(Target, New Rectangle(0, Y, Target.Width, Height))
    End Sub
    Public Shared Sub Off()
        If _HighlightedRect.IsEmpty Then
            Return
        End If
        ControlPaint.FillReversibleRectangle(_HighlightedRect, _AxisColor)
        _HighlightedRect = Rectangle.Empty
    End Sub
End Class

#End Region 'Highlighter