﻿Imports System.Runtime.CompilerServices
Imports System.Globalization.CultureInfo, System.Globalization.CalendarWeekRule
Public Module DatumHelper

#Region "Kalenderwoche"

    Public Function CalendarWeek(Datum As Date) As Integer
        Return CurrentCulture.Calendar.GetWeekOfYear(Datum, FirstFourDayWeek, DayOfWeek.Monday)
    End Function

#End Region 'Kalenderwoche

    Public Function GetDateFromInputBox(Headline As String) As Date
        Using dlg As New dlgDateInput
            If Headline <> "" Then dlg.Text = $"{Headline} eingeben"
            If dlg.ShowDialog = DialogResult.OK Then
                Return Date.Parse(dlg.dtbDate.Text)
            Else
                Return Nothing
            End If
        End Using
    End Function

    ''' <summary> Gibt das Datum des gewünschten Wochentags vor einer Woche zurück </summary>
    Public Function DatumWochentagVergangenheit(dt As Date, alignTo As DayOfWeek) As Date
        Return dt.AddDays(-((dt.DayOfWeek + 7 - alignTo) Mod 7))
    End Function

    ''' <summary> Gibt das Datum des gewünschten Wochentags in der nächsten Woche zurück </summary>
    Public Function DatumWochentagZukunft(dt As Date, alignTo As DayOfWeek) As Date
        Return dt.AddDays(7 + alignTo - dt.DayOfWeek)
    End Function

    ''' <summary> Zählt die Arbeitstage Monatsanfang(EndDate.Month) bis einschl. EndDate. Berücksichtigt werden Wochenenden (Sa, So) und Feiertage </summary>
    Public Function AnzahlArbeitstage_MonatsanfangBisDatumInkl(EndDate As Date, Bundesland As String) As Integer
        Dim cnt = 0
        Dim sDate = MonatsAnfang(EndDate.Month, EndDate.Year)
        Dim eDate = EndDate
        Dim loopDate = sDate
        Do While loopDate < eDate.AddDays(1)
            Dim ft = GetFeiertag(loopDate)
            If loopDate.DayOfWeek <> DayOfWeek.Saturday AndAlso loopDate.DayOfWeek <> DayOfWeek.Sunday Then
                If Not FeiertagBundesland(ft, GetBundesland(Bundesland)) Then
                    cnt += 1 'Wenn Datum kein Wochenende und kein Feiertag, dann hochzählen
                End If
            End If
            loopDate = loopDate.AddDays(1)
        Loop
        Return cnt
    End Function

    ''' <summary> Zählt die Arbeitstage Jahresanfang bis einschl. EndDate. Berücksichtigt werden Wochenenden (Sa, So) und Feiertage </summary>
    Public Function AnzahlArbeitstage_JahresanfangBisDatumInkl(EndDate As Date, Bundesland As String) As Integer
        Dim cnt = 0
        Dim sDate = MonatsAnfang(1, EndDate.Year)
        Dim eDate = EndDate
        Dim loopDate = sDate
        Do While loopDate < eDate.AddDays(1)
            Dim ft = GetFeiertag(loopDate)
            If loopDate.DayOfWeek <> DayOfWeek.Saturday AndAlso loopDate.DayOfWeek <> DayOfWeek.Sunday Then
                If Not FeiertagBundesland(ft, GetBundesland(Bundesland)) Then
                    cnt += 1 'Wenn Datum kein Wochenende und kein Feiertag, dann hochzählen
                End If
            End If
            loopDate = loopDate.AddDays(1)
        Loop
        Return cnt
    End Function

    ''' <summary> Zählt die Arbeitstage eines Monats. Berücksichtigt werden Wochenenden (Sa, So) und Feiertage </summary>
    Public Function AnzahlArbeitstage(Monatnummer As Integer, Jahr As Integer, Bundesland As String) As Integer
        Dim cnt = 0
        Dim sDate = MonatsAnfang(Monatnummer, Jahr)
        Dim eDate = MonatsEnde(Monatnummer, Jahr)
        Dim loopDate = sDate
        Do While loopDate < eDate.AddDays(1)
            Dim ft = GetFeiertag(loopDate)
            If loopDate.DayOfWeek <> DayOfWeek.Saturday AndAlso loopDate.DayOfWeek <> DayOfWeek.Sunday Then
                If Not FeiertagBundesland(ft, GetBundesland(Bundesland)) Then
                    cnt += 1 'Wenn Datum kein Wochenende und kein Feiertag, dann hochzählen
                End If
            End If
            loopDate = loopDate.AddDays(1)
        Loop
        Return cnt
    End Function

    ''' <summary> Excel: überprüft, ob Datum-String Double oder Date ist und returnt den korrekten Wert </summary>
    Public Function ExcelDateOADate(dateString As String) As Date
        Dim Datum As Date = Nothing
        If Not Date.TryParse(dateString, Datum) Then
            Datum = DateTime.FromOADate(ConvertToDouble(dateString))
        Else
            Datum = Date.Parse(dateString)
        End If
        Return Datum
    End Function

    ''' <summary> Gibt den Jahresanfang anhand Jahr zurück </summary>
    Public Function JahresStart(Jahr As Integer) As Date
        Dim d As Date = Date.Parse($"01.01.{Jahr}")
        Return d.Date
    End Function

    ''' <summary> Gibt das Jahresende anhand Jahr zurück </summary>
    Public Function JahresEnde(Jahr As Integer) As Date
        Dim d As Date = Date.Parse($"31.12.{Jahr}")
        Return d.Date
    End Function

    ''' <summary> Gibt den Monatsanfang anhand Monatnummer und Jahr zurück </summary>
    Public Function MonatsAnfang(MonatNummer As Integer, Jahr As Integer) As Date
        Return Date.Parse($"01.{MonatNummer}.{Jahr}").Date
    End Function

    ''' <summary> Gibt das Monatende anhand Monatnummer und Jahr zurück </summary>
    Public Function MonatsEnde(MonatNummer As Integer, Jahr As Integer) As Date
        Return Date.Parse($"01.{MonatNummer}.{Jahr}").AddMonths(1).AddDays(-1).Date
    End Function

    ''' <summary> Gibt den Monatnamen ohne Umlaute anhand Monatnummer zurück </summary>
    Public Function MonatNameOhneUmlaute(monatNummer As Integer) As String
        Dim name = ""
        Select Case monatNummer
            Case 1 : name = "Januar"
            Case 2 : name = "Februar"
            Case 3 : name = "Maerz"
            Case 4 : name = "April"
            Case 5 : name = "Mai"
            Case 6 : name = "Juni"
            Case 7 : name = "Juli"
            Case 8 : name = "August"
            Case 9 : name = "September"
            Case 10 : name = "Oktober"
            Case 11 : name = "November"
            Case 12 : name = "Dezember"
        End Select
        Return name
    End Function

    ''' <summary> Gibt den Monatnamen anhand Monatnummer zurück </summary>
    Public Function MonatName(monatNummer As Integer) As String
        Dim name = ""
        Select Case monatNummer
            Case 1 : name = "Januar"
            Case 2 : name = "Februar"
            Case 3 : name = "März"
            Case 4 : name = "April"
            Case 5 : name = "Mai"
            Case 6 : name = "Juni"
            Case 7 : name = "Juli"
            Case 8 : name = "August"
            Case 9 : name = "September"
            Case 10 : name = "Oktober"
            Case 11 : name = "November"
            Case 12 : name = "Dezember"
        End Select
        Return name
    End Function

    ''' <summary> Gibt den kurzen Monatnamen anhand Monatnummer zurück </summary>
    Public Function MonatNameShort(monatNummer As Integer) As String
        Dim name = ""
        Select Case monatNummer
            Case 1 : name = "Jan"
            Case 2 : name = "Feb"
            Case 3 : name = "Mär"
            Case 4 : name = "Apr"
            Case 5 : name = "Mai"
            Case 6 : name = "Jun"
            Case 7 : name = "Jul"
            Case 8 : name = "Aug"
            Case 9 : name = "Sep"
            Case 10 : name = "Okt"
            Case 11 : name = "Nov"
            Case 12 : name = "Dez"
            Case Else : msgError($"{monatNummer} ist keine gültige Monatsnummer!") : Return ""
        End Select
        Return name
    End Function

    ''' <summary> Gibt die Monatnummer anhand Monatname zurück </summary>
    Public Function MonatNummer(monatName As String) As Integer
        Dim month = 0
        Select Case monatName
            Case "Januar" : month = 1
            Case "Februar" : month = 2
            Case "März" : month = 3
            Case "April" : month = 4
            Case "Mai" : month = 5
            Case "Juni" : month = 6
            Case "Juli" : month = 7
            Case "August" : month = 8
            Case "September" : month = 9
            Case "Oktober" : month = 10
            Case "November" : month = 11
            Case "Dezember" : month = 12
            Case Else : msgError($"{monatName} ist kein gültiger Monatsname!") : Return 0
        End Select
        Return month
    End Function

    Public Function MonatSelect(MonatList As List(Of MonatSelection)) As List(Of MonatSelection)
        Dim AlleMonate As New List(Of MonatSelection)
        Dim SelectedMonate As New List(Of MonatSelection)
        For i = 1 To 12
            AlleMonate.Add(New MonatSelection With {.Name = MonatName(i), .Nr = i})
        Next
        Using dlg As New dlgChkLb
            dlg.Text = "Monat(e) auswählen"
            dlg.chkLb.DataSource = AlleMonate
            dlg.chkLb.ValueMember = "Name"
            For i = 0 To dlg.chkLb.Items.Count - 1
                Dim itm = DirectCast(dlg.chkLb.Items(i), MonatSelection)
                If MonatList Is Nothing OrElse MonatList.Count = 0 Then 'Wenn noch keine Selektion vorhanden, dann alle markieren
                    dlg.chkLb.SetItemChecked(i, True)
                Else 'vorherige Selektion wiederherstellen
                    If MonatList.Contains(itm) Then dlg.chkLb.SetItemChecked(i, True)
                End If
            Next
            If dlg.ShowDialog() <> DialogResult.OK Then Return Nothing
            For Each itm In dlg.chkLb.CheckedItems
                SelectedMonate.Add(DirectCast(itm, MonatSelection))
            Next
        End Using
        Return SelectedMonate
    End Function

#Region "Feiertage und Bundesländer"
    Public Enum Feiertag
        'Monate sind eincodiert an Bit-Positionen >= 16 ( '<<' Bitshift-Operator)
        KeinFeiertag = 0
        Neujahr = 1 + (1 << 16)                       'FEST (01.01.) -> alle BL
        HeiligeDreiKönige = 6 + (1 << 16)             'FEST (06.01.) -> BW,BY,ST
        'InternationalerFrauentag                     ' -> BE
        TagDerArbeit = 1 + (5 << 16)                  'FEST (01.05.) -> alle BL
        MariäHimmelfahrt = 15 + (8 << 16)             'FEST (15.08.) -> BY,SL
        'Weltkindertag                                ' -> TH
        TagDerDeutschenEinheit = 3 + (10 << 16)       'FEST (03.10.) -> alle BL
        Reformationstag = 31 + (10 << 16)             'FEST (31.10.) -> BB,HB,HH,MV,NI,SN,ST,SH,TH
        Allerheiligen = 1 + (11 << 16)                'FEST (01.11.) -> BW,BY,NW,RP,SL
        ErsterWeihnachtsfeiertag = 25 + (12 << 16)    'FEST (25.12.) -> alle BL
        ZweiterWeihnachtsfeiertag = 26 + (12 << 16)   'FEST (26.12.) -> alle BL
        'Bewegliche Feiertage (und None) haben keinen Monat eincodiert
        'Beachte, dass Enum-Werte sich fortlaufend erhöhen, wenn nicht explizit gesetzt
        Karfreitag = 2                                'BEWEGLICH -> alle BL
        Ostersonntag                                  'BEWEGLICH -> BB
        Ostermontag                                   'BEWEGLICH -> alle BL
        ChristiHimmelfahrt                            'BEWEGLICH -> alle BL
        PfingstSonntag                                'BEWEGLICH -> BB
        PfingstMontag                                 'BEWEGLICH -> alle BL
        Fronleichnam                                  'BEWEGLICH -> BW,BY,HE,NW,RP,SL
        BussUndBettag                                 'BEWEGLICH -> SN
    End Enum

    Private _Feiertag As New HashSet(Of Integer)(DirectCast([Enum].GetValues(GetType(Feiertag)), Integer()))
    <Extension()>
    Public Function GetFeiertag(ByVal dtDate As Date) As Feiertag
        dtDate = dtDate.Date
        Dim n = dtDate.Day + (dtDate.Month << 16)
        If _Feiertag.Contains(n) Then Return DirectCast(n, Feiertag)
        Dim Ostersonntag = GetOstersonntag(dtDate.Year)
        Dim BussUndBettag = GetBussUndBettag(dtDate.Year)
        If dtDate = Ostersonntag.AddDays(-2) Then Return Feiertag.Karfreitag
        If dtDate = Ostersonntag.AddDays(1) Then Return Feiertag.Ostermontag
        If dtDate = Ostersonntag Then Return Feiertag.Ostersonntag
        If dtDate = Ostersonntag.AddDays(39) Then Return Feiertag.ChristiHimmelfahrt
        If dtDate = Ostersonntag.AddDays(49) Then Return Feiertag.PfingstSonntag
        If dtDate = Ostersonntag.AddDays(50) Then Return Feiertag.PfingstMontag
        If dtDate = Ostersonntag.AddDays(60) Then Return Feiertag.Fronleichnam
        If dtDate = BussUndBettag Then Return Feiertag.BussUndBettag
        Return Feiertag.KeinFeiertag
    End Function

    Private Function GetOstersonntag(ByVal year As Integer) As Date
        Dim a As Integer = year Mod 19
        Dim b As Integer = year \ 100
        Dim c As Integer = (8 * b + 13) \ 25 - 2
        Dim d As Integer = b - (year \ 400) - 2
        Dim e As Integer = (19 * (year Mod 19) + ((15 - c + d) Mod 30)) Mod 30
        If e = 28 Then
            If a > 10 Then
                e = 27
            End If
        ElseIf e = 29 Then
            e = 28
        End If
        Dim f As Integer = (d + 6 * e + 2 * (year Mod 4) + 4 * (year Mod 7) + 6) Mod 7
        Return New DateTime(year, 3, 22).AddDays(e + f)
    End Function

    Private Function GetBussUndBettag(ByVal year As Integer) As Date
        Dim datum = New DateTime(year, 12, 25)
        Dim day As Integer = CInt(datum.DayOfWeek)
        day = If((day = 0), -11, -day - 4)
        datum = datum.AddDays(-4 * 7).AddDays(day)
        Return datum
    End Function

    <Flags>
    Public Enum Bundesland
        None = 0
        SH = &H1
        HH = &H2
        NI = &H4
        HB = &H8
        NW = &H10
        HE = &H20
        RP = &H40
        BW = &H80
        BY = &H100
        SL = &H200
        BE = &H400
        BB = &H800
        MV = &H1000
        SN = &H2000
        ST = &H4000
        TH = &H8000
    End Enum

    Public Function GetBundesland(ByVal blName As String) As Bundesland
        Dim bl As Bundesland
        Select Case blName
            Case "Schleswig-Holstein" : bl = Bundesland.SH
            Case "Hamburg" : bl = Bundesland.HH
            Case "Niedersachsen" : bl = Bundesland.NI
            Case "Bremen" : bl = Bundesland.HB
            Case "Nordrhein-Westfalen" : bl = Bundesland.NW
            Case "Hessen" : bl = Bundesland.HE
            Case "Rheinland-Pfalz" : bl = Bundesland.RP
            Case "Baden-Württemberg" : bl = Bundesland.BW
            Case "Bayern" : bl = Bundesland.BY
            Case "Saarland" : bl = Bundesland.SL
            Case "Berlin" : bl = Bundesland.BE
            Case "Brandenburg" : bl = Bundesland.BB
            Case "Mecklenburg-Vorpommern" : bl = Bundesland.MV
            Case "Sachsen" : bl = Bundesland.SN
            Case "Sachsen-Anhalt" : bl = Bundesland.ST
            Case "Thüringen" : bl = Bundesland.TH
        End Select
        Return bl
    End Function

    Public Function FeiertagBundesland(ByVal ft As Feiertag, Optional ByVal bl As Bundesland = 0) As Boolean
        Dim result = False
        Select Case ft
            Case Feiertag.HeiligeDreiKönige
                result = bl.HasFlag(Bundesland.BW) OrElse bl.HasFlag(Bundesland.BY) OrElse bl.HasFlag(Bundesland.ST)
            Case Feiertag.Fronleichnam
                result = bl.HasFlag(Bundesland.BW) OrElse bl.HasFlag(Bundesland.BY) OrElse bl.HasFlag(Bundesland.HE) OrElse bl.HasFlag(Bundesland.NW) OrElse bl.HasFlag(Bundesland.RP) OrElse bl.HasFlag(Bundesland.SL)
            Case Feiertag.MariäHimmelfahrt
                result = bl.HasFlag(Bundesland.BY) OrElse bl.HasFlag(Bundesland.SL)
            Case Feiertag.Reformationstag
                result = bl.HasFlag(Bundesland.BB) OrElse bl.HasFlag(Bundesland.HB) OrElse bl.HasFlag(Bundesland.HH) OrElse bl.HasFlag(Bundesland.MV) OrElse bl.HasFlag(Bundesland.NI) OrElse bl.HasFlag(Bundesland.SN) OrElse bl.HasFlag(Bundesland.ST) OrElse bl.HasFlag(Bundesland.SH) OrElse bl.HasFlag(Bundesland.TH)
            Case Feiertag.Allerheiligen
                result = bl.HasFlag(Bundesland.BW) OrElse bl.HasFlag(Bundesland.BY) OrElse bl.HasFlag(Bundesland.NW) OrElse bl.HasFlag(Bundesland.RP) OrElse bl.HasFlag(Bundesland.SL)
            Case Feiertag.Ostersonntag, Feiertag.PfingstSonntag
                result = bl.HasFlag(Bundesland.BB)
            Case Feiertag.BussUndBettag
                result = bl.HasFlag(Bundesland.SN)
            Case Feiertag.Neujahr, Feiertag.TagDerArbeit, Feiertag.TagDerDeutschenEinheit, Feiertag.ErsterWeihnachtsfeiertag,
                 Feiertag.ZweiterWeihnachtsfeiertag, Feiertag.Karfreitag, Feiertag.Ostermontag, Feiertag.ChristiHimmelfahrt, Feiertag.PfingstMontag
                result = True
        End Select
        Return result
    End Function
#End Region 'Feiertage und Bundesländer

End Module

Public Class MonatSelection
    Property Name As String
    Property Nr As Integer
End Class
