﻿Imports System.Text

''' <summary>
''' Provides basic operations for writing ini content into streams.
''' </summary>
Public Class IniStreamWriter
    Inherits IniTextWriter

    Private strmUnderlyingStream As IO.Stream
    Private encEncoding As Encoding
    Private btBuffer() As Byte
    Private intBufferPosition As Integer

    ''' <summary>
    ''' Writes the specified string into the underlying stream.
    ''' </summary>
    ''' <param name="value">The string to write into the stream.</param>
    Protected Overrides Sub Write(ByVal value As String)
        Dim byteCount As Integer = encEncoding.GetByteCount(value)
        If intBufferPosition + byteCount > btBuffer.Length Then
            Dim bts As Integer = encEncoding.GetBytes(value, 0, value.Length, btBuffer, intBufferPosition)
            Dim chs As Integer = encEncoding.GetCharCount(btBuffer, intBufferPosition, bts)

            strmUnderlyingStream.Write(btBuffer, 0, btBuffer.Length)
            While value.Length > chs
                bts = encEncoding.GetBytes(value, chs, value.Length - chs, btBuffer, 0)
                chs += encEncoding.GetCharCount(btBuffer, 0, bts)
                strmUnderlyingStream.Write(btBuffer, 0, bts)
            End While
        Else
            intBufferPosition += encEncoding.GetBytes(value, 0, value.Length, btBuffer, intBufferPosition)
        End If
    End Sub

    ''' <summary>
    ''' Writes the specified characters into the underlying stream.
    ''' </summary>
    ''' <param name="value">The characters to write into the stream.</param>
    Protected Overrides Sub Write(ByVal ParamArray value() As Char)
        Dim byteCount As Integer = encEncoding.GetByteCount(value)
        If intBufferPosition + byteCount > btBuffer.Length Then
            Dim bts As Integer = encEncoding.GetBytes(value, 0, value.Length, btBuffer, intBufferPosition)
            Dim chs As Integer = encEncoding.GetCharCount(btBuffer, intBufferPosition, bts)

            strmUnderlyingStream.Write(btBuffer, 0, btBuffer.Length)
            While value.Length > chs
                bts = encEncoding.GetBytes(value, chs, value.Length - chs, btBuffer, 0)
                chs += encEncoding.GetCharCount(btBuffer, 0, bts)
                strmUnderlyingStream.Write(btBuffer, 0, bts)
            End While
        Else
            intBufferPosition += encEncoding.GetBytes(value, 0, value.Length, btBuffer, intBufferPosition)
        End If
    End Sub

    ''' <summary>
    ''' Writes all buffered contents into the underlying stream.
    ''' </summary>
    Public Overrides Sub Flush()
        strmUnderlyingStream.Write(btBuffer, 0, intBufferPosition)
        intBufferPosition = 0
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the IniLib.IniStreamWriter class using the specified stream as the underlying stream.
    ''' </summary>
    ''' <param name="underlyingStream">Specifies the stream from which to read.</param>
    Public Sub New(ByVal underlyingStream As IO.Stream)
        strmUnderlyingStream = underlyingStream
        encEncoding = Encoding.Default
        btBuffer = New Byte(32767) {}
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the IniLib.IniStreamWriter class using the specified stream as the underlying stream that is encoded as specified.
    ''' </summary>
    ''' <param name="underlyingStream">Specifies the stream from which to read.</param>
    ''' <param name="encoding">Specifies the encoding of the underlying stream.</param>
    Public Sub New(ByVal underlyingStream As IO.Stream, ByVal encoding As Encoding)
        strmUnderlyingStream = underlyingStream
        encEncoding = encoding
        btBuffer = New Byte(32767) {}
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the IniLib.IniStreamWriter class using the specified stream as the underlying stream that is encoded as specified and using a byte buffer with the specified buffer size.
    ''' </summary>
    ''' <param name="underlyingStream">Specifies the stream from which to read.</param>
    ''' <param name="encoding">Specifies the encoding of the underlying stream.</param>
    ''' <param name="bufferSize">Specifies the size of the byte buffer.</param>
    Public Sub New(ByVal underlyingStream As IO.Stream, ByVal encoding As Encoding, ByVal bufferSize As Integer)
        strmUnderlyingStream = underlyingStream
        encEncoding = encoding
        btBuffer = New Byte(bufferSize - 1) {}
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the IniLib.IniStreamWriter class using the specified stream as the underlying stream and using a byte buffer with the specified buffer size.
    ''' </summary>
    ''' <param name="underlyingStream">Specifies the stream from which to read.</param>
    ''' <param name="bufferSize">Specifies the size of the byte buffer.</param>
    Public Sub New(ByVal underlyingStream As IO.Stream, ByVal bufferSize As Integer)
        strmUnderlyingStream = underlyingStream
        encEncoding = Encoding.Default
        btBuffer = New Byte(bufferSize - 1) {}
    End Sub
End Class
