﻿Imports System.Drawing, System.Drawing.Printing

Public Module Print

    Private WithEvents _PrintDoc As PrintDocument
    Private _dgv As DataGridView
    Private _myfont As Font = New Font("Arial", 11, FontStyle.Regular)
    Private _penGray As New Pen(Brushes.LightGray, 0.5)
    Private _headerHeight As Integer = 30
    Private _rowHeight As Integer = 20
    Private _margins As Integer = 40
    Private _UsedWidth As Integer
    Private _UsedHeight As Integer
    Private _startColIndex As Integer
    Private _activeColIndex As Integer
    Private _interruptColIndex As Integer
    Private _printedMaxColIndex As Integer
    Private _activeRowIndex As Integer
    Private _hasRowInterruption As Boolean = False
    Private _hasColInterruption As Boolean = False
    Private _pageCounter As Integer = 1

    'TODO: Abfangen, wenn Headertext mehrzeilig ist dann soll er auch mehrzeilig gedruckt werden

    Public Sub PrintDgv(DocumentName As String, dgv As DataGridView)
        _PrintDoc = New PrintDocument With {.DocumentName = DocumentName}
        _dgv = dgv
        ResetVars()
        Using dlgPreview As New dlgPrintPreview
            dlgPreview.WindowState = FormWindowState.Maximized
            dlgPreview.PrintPreviewControl1.Document = _PrintDoc
            dlgPreview.nudHorizontal.Value = 1 'TODO: nur zum Testen, gehört in den Dialog selbst
            dlgPreview.nudVertical.Value = 1 'TODO: nur zum Testen, gehört in den Dialog selbst
            dlgPreview.ShowDialog()
        End Using
    End Sub

    Private Sub ResetVars()
        _UsedWidth = 0
        _UsedHeight = 0
        _startColIndex = 0
        _activeColIndex = 0
        _interruptColIndex = 0
        _printedMaxColIndex = 0
        _activeRowIndex = 0
        _hasRowInterruption = False
        _hasColInterruption = False
        _pageCounter = 1 'TODO: Wenn Seitenansicht gewechselt wird, dann zählt er einfach weiter. Muss dann resettet werden
    End Sub

    Private Sub PrintDoc_PrintPage(sender As Object, e As PrintPageEventArgs) Handles _PrintDoc.PrintPage
        Dim g = e.Graphics
        Dim margin = _PrintDoc.DefaultPageSettings.Margins
        margin.Left = _margins
        margin.Right = _margins
        margin.Top = _margins
        margin.Bottom = _margins
        Dim X = margin.Left
        Dim Y = margin.Top
        g.DrawString($"{_PrintDoc.DocumentName} | Seite: {_pageCounter}", _myfont, Brushes.Black, X, Y) 'Überschrift
        Y += 30 'etwas Platz schaffen zu den Spaltenköpfen
        Dim kvpOfColsAndMaxWidth = GetVisibleColAndMaxWidth() 'Spalten und deren maximale Breiten holen 'TODO: klappt nicht einwandfrei, Text steht teilweise trotzdem über
        Dim maxPrintWidth = If(_PrintDoc.DefaultPageSettings.Landscape, _PrintDoc.DefaultPageSettings.PaperSize.Height, _PrintDoc.DefaultPageSettings.PaperSize.Width) - margin.Right - margin.Left 'die maximale Breite je nach Orientierung
        e.HasMorePages = False 'wenn weiter unten weitere Seiten gefordert wurden, dann hier zurücksetzen - sonst Endlosschleife
        'Columns
        If _hasRowInterruption Then
            _startColIndex = _printedMaxColIndex + 1
            _activeColIndex = _printedMaxColIndex + 1
        Else
            _startColIndex = If(_hasColInterruption, _interruptColIndex, 0)
            _activeColIndex = If(_hasColInterruption, _interruptColIndex, 0)
        End If
        For iCol = _activeColIndex To _dgv.Columns.Count - 1
            If Not _hasRowInterruption Then _hasColInterruption = False 'solange wie es Row-Unterbrechungen gibt muss/soll er die Spaltenköpfe auf jeder Seite neu malen 'TODO: Einstellbar machen?
            _activeColIndex = iCol
            Dim colIsVisible = kvpOfColsAndMaxWidth.Any(Function(kvp) kvp.Key.Index = _activeColIndex)
            If Not colIsVisible Then Continue For 'wenn die Spalte im DGV nicht sichtbar ist, dann überspringen
            Dim colAndMaxWidth = kvpOfColsAndMaxWidth.First(Function(kvp) kvp.Key.Index = _activeColIndex) 'passendes KVP raussuchen



            'TODO: Bei Fahrzeug testen, wenn der Dialog umgebaut ist. Im Hochformat klappt das, im Querformat macht er unnötige leere Seiten dazwischen...
            If colAndMaxWidth.Value > maxPrintWidth Then 'Workaround, falls eine Col mal breiter sein sollte als die Seite, dann muss trotzdem gedruckt werden
                If _UsedWidth <> 0 Then 'allerdings auf neuer Seite
                    _hasColInterruption = True
                    _interruptColIndex = _activeColIndex
                    Exit For
                End If
            Else
                If _UsedWidth + colAndMaxWidth.Value > maxPrintWidth Then 'überschreiten die Spaltenköpfe die Seitenbreite, dann Abbruch und erstmal Rows eintragen
                    _hasColInterruption = True
                    _interruptColIndex = _activeColIndex 'Abbruch-Spaltenindex merken
                    Exit For
                End If
            End If
            DrawHeader(g, X, Y, _UsedWidth, colAndMaxWidth) 'Spaltenkopf zeichnen
            _UsedWidth += colAndMaxWidth.Value
        Next
        _UsedHeight = Y + _headerHeight + 5 'etwas Platz zwischen Spaltenköpfe und Rows schaffen
        _UsedWidth = 0 'Benutzte Breite zurücksetzen, damit die Rows nun wieder von ganz links an gemalt werden
        'Rows
        Dim toColIndex = If(_hasColInterruption, _activeColIndex - 1, _activeColIndex) 'Wenn ein Abbruch wegen Überbreite (Spalten) erfolgt, dann activeColIndex-1, sonst wird bei den Rows 1 Spalteninhalt zuviel gemalt.
        DrawRows(g, X, _startColIndex, toColIndex, kvpOfColsAndMaxWidth) 'die zugehörigen Row-Inhalte der bisher gemalten Spaltenköpfe
        If _hasRowInterruption Then
            e.HasMorePages = True
            _pageCounter += 1
            Return
        Else
            _printedMaxColIndex = toColIndex 'Wenn kein Row-Abbruch mehr kommt, dann hier den Max-Index der bisher gemalten Spaltenköpfe merken
        End If
        If _hasColInterruption Then
            e.HasMorePages = True
            _pageCounter += 1
            Return
        End If
        e.HasMorePages = False
    End Sub

    Private Sub DrawRows(g As Graphics, X As Integer, startCol As Integer, endCol As Integer, kvpOfColsAndMaxWidth As Dictionary(Of DataGridViewColumn, Integer))
        Dim margin = _PrintDoc.DefaultPageSettings.Margins
        Dim maxPrintHeight = If(_PrintDoc.DefaultPageSettings.Landscape, _PrintDoc.DefaultPageSettings.PaperSize.Width, _PrintDoc.DefaultPageSettings.PaperSize.Height) - margin.Bottom 'die maximale Höhe
        For iRow = 0 To _dgv.Rows.Count - 1
            If _UsedHeight + _rowHeight > maxPrintHeight Then
                _hasRowInterruption = True
                Exit For
            End If
            If Not _hasRowInterruption Then _activeRowIndex = iRow
            If iRow <> _activeRowIndex Then Continue For
            _hasRowInterruption = False
            Dim rw = _dgv.Rows(_activeRowIndex)
            For iCol = startCol To endCol
                Dim colIndex = iCol
                Dim colIsVisible = kvpOfColsAndMaxWidth.Any(Function(kvp) kvp.Key.Index = colIndex)
                If Not colIsVisible Then Continue For
                Dim colAndMaxWidth = kvpOfColsAndMaxWidth.First(Function(kvp) kvp.Key.Index = colIndex)
                Dim formattedValue = rw.Cells(iCol).FormattedValue.ToString
                If formattedValue = "True" Then formattedValue = "Ja"
                If formattedValue = "False" Then formattedValue = "Nein"
                g.DrawString(formattedValue, _myfont, Brushes.Black, X + _UsedWidth, _UsedHeight + 1)
                g.DrawRectangle(_penGray, X + _UsedWidth, _UsedHeight, colAndMaxWidth.Value, _rowHeight)
                _UsedWidth += colAndMaxWidth.Value
            Next
            _UsedWidth = 0
            _UsedHeight += _rowHeight
        Next
    End Sub

    Private Sub DrawHeader(g As Graphics, x As Integer, y As Integer, totalWidth As Integer, colAndMaxWidth As KeyValuePair(Of DataGridViewColumn, Integer))
        Dim drawHeaderText As Action(Of String, Integer) = Sub(s, dx) g.DrawString(s, _myfont, Brushes.Black, x + dx, y + 5)
        g.FillRectangle(Brushes.LightGray, x + totalWidth, y, colAndMaxWidth.Value, _headerHeight)
        drawHeaderText(colAndMaxWidth.Key.HeaderText, totalWidth)
    End Sub

    ''' <summary>Speichert alle sichtbaren DGV-Columns, sowie die maximale Schriftbreite der Spalte (oder HeaderCell-Breite, falls größer) in einem Dictionary</summary>
    Private Function GetVisibleColAndMaxWidth() As Dictionary(Of DataGridViewColumn, Integer)
        Dim dic As New Dictionary(Of DataGridViewColumn, Integer)
        Dim g = _dgv.CreateGraphics
        For i = 0 To _dgv.ColumnsX.Where(Function(col) col.Visible).Count - 1
            Dim colIndex = i
            Dim ht = _dgv.Columns(colIndex).HeaderText
            Dim CellContent = _dgv.RowsX.Select(Function(rw) rw.Cells(colIndex).FormattedValue.ToString).ToArray
            Dim maxCellContentWidth = g.MeasureString(GetLongestString(CellContent), _myfont).ToSize.Width 'TODO: klappt noch nicht zu 100%
            Dim colWidth = g.MeasureString(_dgv.Columns(i).HeaderText, _myfont).ToSize.Width
            If colWidth > maxCellContentWidth Then
                dic.Add(_dgv.Columns(colIndex), colWidth + 5)
            Else
                dic.Add(_dgv.Columns(colIndex), maxCellContentWidth + 5)
            End If
        Next
        Return dic
    End Function

End Module
